@@ -8,7 +8,6 @@ Customization for the Solaris operation system.
 """
 
 import socket
-import scapy.consts
 
 from scapy.config import conf
 conf.use_pcap = True
@@ -19,9 +18,9 @@ socket.IPPROTO_GRE = 47
 # From sys/sockio.h and net/if.h
 SIOCGIFHWADDR = 0xc02069b9  # Get hardware address
 
-from scapy.arch.pcapdnet import *  # noqa: F401, F403
-from scapy.arch.unix import *  # noqa: F401, F403
-from scapy.arch.common import get_if_raw_hwaddr  # noqa: F401, F403
+from scapy.arch.pcapdnet import *  # noqa: F401, F403, E402
+from scapy.arch.unix import *  # noqa: F401, F403, E402
+from scapy.arch.common import get_if_raw_hwaddr  # noqa: F401, F403, E402
 
 
 def get_working_if():
@@ -32,5 +31,5 @@ def get_working_if():
         iface = min(conf.route.routes, key=lambda x: x[1])[3]
     except ValueError:
         # no route
-        iface = scapy.consts.LOOPBACK_INTERFACE
+        iface = conf.loopback_name
     return iface
