@@ -112,7 +112,7 @@ DHCPTypes = {
 DHCPOptions = {
     0: "pad",
     1: IPField("subnet_mask", "0.0.0.0"),
-    2: "time_zone",
+    2: IntField("time_zone", 500),
     3: IPField("router", "0.0.0.0"),
     4: IPField("time_server", "0.0.0.0"),
     5: IPField("IEN_name_server", "0.0.0.0"),
@@ -132,9 +132,9 @@ DHCPOptions = {
     19: ByteField("ip-forwarding", 0),
     20: ByteField("non-local-source-routing", 0),
     21: IPField("policy-filter", "0.0.0.0"),
-    22: "max_dgram_reass_size",
-    23: "default_ttl",
-    24: "pmtu_timeout",
+    22: ShortField("max_dgram_reass_size", 300),
+    23: ByteField("default_ttl", 50),
+    24: IntField("pmtu_timeout", 1000),
     25: ShortField("path-mtu-plateau-table", 1000),
     26: ShortField("interface-mtu", 50),
     27: ByteField("all-subnets-local", 0),
@@ -145,12 +145,12 @@ DHCPOptions = {
     32: IPField("router-solicitation-address", "0.0.0.0"),
     33: IPField("static-routes", "0.0.0.0"),
     34: ByteField("trailer-encapsulation", 0),
-    35: "arp_cache_timeout",
-    36: "ether_or_dot3",
-    37: "tcp_ttl",
-    38: "tcp_keepalive_interval",
-    39: "tcp_keepalive_garbage",
-    40: "NIS_domain",
+    35: IntField("arp_cache_timeout", 1000),
+    36: ByteField("ieee802-3-encapsulation", 0),
+    37: ByteField("tcp_ttl", 100),
+    38: IntField("tcp_keepalive_interval", 1000),
+    39: ByteField("tcp_keepalive_garbage", 0),
+    40: StrField("NIS_domain", "www.example.com"),
     41: IPField("NIS_server", "0.0.0.0"),
     42: IPField("NTP_server", "0.0.0.0"),
     43: "vendor_specific",
@@ -170,8 +170,8 @@ DHCPOptions = {
     57: ShortField("max_dhcp_size", 1500),
     58: IntField("renewal_time", 21600),
     59: IntField("rebinding_time", 37800),
-    60: "vendor_class_id",
-    61: "client_id",
+    60: StrField("vendor_class_id", "id"),
+    61: StrField("client_id", ""),
     62: "nwip-domain-name",
     64: "NISplus_domain",
     65: IPField("NISplus_server", "0.0.0.0"),
@@ -184,12 +184,45 @@ DHCPOptions = {
     73: IPField("Finger_server", "0.0.0.0"),
     74: IPField("IRC_server", "0.0.0.0"),
     75: IPField("StreetTalk_server", "0.0.0.0"),
-    76: "StreetTalk_Dir_Assistance",
+    76: IPField("StreetTalk_Dir_Assistance", "0.0.0.0"),
+    78: "slp_service_agent",
+    79: "slp_service_scope",
     81: "client_FQDN",
-    82: "relay_agent_Information",
+    82: "relay_agent_information",
+    85: IPField("nds-server", "0.0.0.0"),
+    86: StrField("nds-tree-name", ""),
+    87: StrField("nds-context", ""),
+    88: "bcms-controller-namesi",
+    89: IPField("bcms-controller-address", "0.0.0.0"),
     91: IntField("client-last-transaction-time", 1000),
     92: IPField("associated-ip", "0.0.0.0"),
+    93: "pxe_client_architecture",
+    94: "pxe_client_network_interface",
+    97: "pxe_client_machine_identifier",
+    98: StrField("uap-servers", ""),
+    100: StrField("pcode", ""),
+    101: StrField("tcode", ""),
+    112: IPField("netinfo-server-address", "0.0.0.0"),
+    113: StrField("netinfo-server-tag", ""),
+    114: StrField("default-url", ""),
+    116: ByteField("auto-config", 0),
+    117: ShortField("name-service-search", 0,),
     118: IPField("subnet-selection", "0.0.0.0"),
+    124: "vendor_class",
+    125: "vendor_specific_information",
+    136: IPField("pana-agent", "0.0.0.0"),
+    137: "v4-lost",
+    138: IPField("capwap-ac-v4", "0.0.0.0"),
+    141: "sip_ua_service_domains",
+    146: "rdnss-selection",
+    159: "v4-portparams",
+    160: StrField("v4-captive-portal", ""),
+    208: "pxelinux_magic",
+    209: "pxelinux_configuration_file",
+    210: "pxelinux_path_prefix",
+    211: "pxelinux_reboot_time",
+    212: "option-6rd",
+    213: "v4-access-domain",
     255: "end"
 }
 
